package org.apache.maven.plugin.coreit;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugins.annotations.Component;
import org.apache.maven.plugins.annotations.ResolutionScope;
import org.apache.maven.plugins.annotations.Execute;
import org.apache.maven.plugins.annotations.InstantiationStrategy;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.apache.maven.project.MavenProjectHelper;

/**
 * Checks maximum annotations with non-default values.
 *
 * @since since-text
 * @deprecated deprecated-text
 */
@Mojo( name = "maximal",
    aggregator = true,
    configurator = "configurator-hint",
    requiresDependencyResolution = ResolutionScope.COMPILE,
    requiresDependencyCollection = ResolutionScope.TEST,
    defaultPhase = LifecyclePhase.PACKAGE,
    executionStrategy = "always",
    instantiationStrategy = InstantiationStrategy.SINGLETON,
    inheritByDefault = false,
    requiresDirectInvocation = true,
    requiresOnline = true,
    requiresProject = false,
    requiresReports = true,
    threadSafe = true )
@Execute( phase = LifecyclePhase.COMPILE )
public class Maximal
    extends AbstractMojo
{
    /**
     * Parameter description.
     *
     * @since since-text
     * @deprecated deprecated-text
     */
    @Parameter( alias = "myAlias",
        property = "aProperty",
        defaultValue = "${anExpression}",
        readonly = true,
        required = true )
    private String param;

    @Component( role = MavenProjectHelper.class, hint = "test" )
    private Object projectHelper;

    public void execute()
    {
    }

}
